(document:surround "/std/frame")

(define (ui-init)
  (catch/message
    (lambda()
      (form-update-enum "all_rules" (woo-list "/audit/list_all_rules"))
      (form-update-enum "rule_list" (woo-list "/audit/list_rules"))
      (form-update-enum "condition" (woo-list "/audit/list_all_conditions"))
      (form-update-enum "filters" (woo-list "/audit/list_all_filters" ))
     )
  )
  (clean)
)

(define (config_rule)
  (clean)
  (if (not (equal? (rule_all current) -1)) (configure_rule)))

(define (configure_rule)
  (let ((data (woo-read-first "/audit/config_rule" 'rule (form-value "all_rules"))))
    (rule_name value ( if (equal? (rule_all text) (form-value "all_rules")) "" (rule_all text) ))
    (form-update-value "active" (equal? (car(rule_all row-item (list (rule_all current) 1))) "on") )
    (form-update-value-list '("rule_action") data)
    (form-update-value-list '("rule_list" "rule_syscall") data)
    (form-update-enum "filter_list" (woo-list "/audit/list_filters" 'rule (form-value "all_rules")))
    (form-update-value-list '("perm_r" "perm_w" "perm_x" "perm_a" "check_file") data)
    (path_dir_file value (woo-get-option data 'path_file))
  )
  (dir_or_file)
)

(define (clean)
  (let ((data (woo-read "/audit/clean_all")))
    (form-update-value-list '("rule_action" "filters" "condition" "filter_val" "rule_list" "name") data) )
  (fil_list rows-clear)
  (form-update-visibility "change_filter" #f)
  (form-update-visibility "delete_filter" #f)
  (form-update-visibility "clean_filter_list" #f)
  (dir_or_file)
)

(define (config_filter)
  (form-update-value "filters" (fil_list text))
  (form-update-value "condition" (car(fil_list row-item (list (fil_list current) 1))))
  (fil_val value (car(fil_list row-item (list (fil_list current) 2))))
  (form-update-visibility "change_filter" #t)
  (form-update-visibility "delete_filter" #t)
  (form-update-visibility "clean_filter_list" #t)
)

(define (change_filter)
  (fil_list row-item (fil_list current) (list (fil text)))
  (fil_list row-item (list (fil_list current) 1) (list (fil_check text)))
  (fil_list row-item (list (fil_list current) 2) (fil_val text))
)

(define (add_filter)
  (if (and (not (equal? (fil value) ""))
	   (not (equal? (fil_check value) ""))
	   (not (equal? (fil_val value)"")) )
    (fil_list append-row (vector (fil text) (fil_check text) (fil_val text)) ))
)

(define (activate)
  (rule_all row-item (list (rule_all current) 1) ( if (form-value "active") (list "on") (list "") ))
  (woo-call "/audit/activate_rule" 'stat (form-value "active")
	    			   'num (rule_all current))
  (if (not (equal? (rule_all current) -1)) (form-update-visibility "reload" #t))
)

(define (permission)
  (vector
    (if (form-value "perm_r") "r" "")
    (if (form-value "perm_w") "w" "")
    (if (form-value "perm_x") "x" "")
    (if (form-value "perm_a") "a" "")
  )
)

(define (new_rule mode)
  (define (extract-text0 x) (car (vector-ref x 0)))
  (define (extract-text1 x) (car (vector-ref x 1)))
  (define (extract-text2 x) (car (vector-ref x 2)))
  (catch/message
   (lambda()
    (woo-write "/audit/new_rule" 'syscall (form-value "rule_syscall")
	    			 'rule (form-value "rule_list")
				 'action (form-value "rule_action")
				 'all_rules (form-value "all_rules")
				 'add_rule mode
				 'name (form-value "name")
				 'num (rule_all current)
				 'count (rule_all count)
				 'expert (form-value "expert_mode")
				 'path (path_dir_file value)
				 'perm (permission)

	     			 'filter_name (map extract-text0 (fil_list rows))
	     			 'filter_val (map extract-text2 (fil_list rows))
	     			 'filter_cond (map extract-text1 (fil_list rows))
				 )
    )
;    (document:popup-warning (_ "msg") #t)
;    (lambda(msg) (document:popup-critical (_ "msg") 'ok) #t)
    )
  (form-update-enum "all_rules" (woo-list "/audit/list_all_rules"))
; (clean)
)

(define (delete_rule)
  (catch/message
    (lambda()
      (woo-call "/audit/delete_rule"
		'num  (rule_all current)
		'rule (form-value "all_rules")
		)
  (form-update-enum "all_rules" (woo-list "/audit/list_all_rules"))
  (clean)))
)

(define (turn_rule)
  (list (list (rule_all row-item (rule_all current)) (list (rule_all row-item (+(rule_all current) 1))) ))
)

(define (template)
  (form-popup "/audit/rules/templates")
  (ui-init)
)

(define (expert)
  (define mode (form-value "expert_mode"))
  (form-update-visibility "rule_syscall" mode)
  (form-update-visibility "label_syscall" mode)
  (form-update-visibility "label_rule" mode)
  (form-update-visibility "rule_list" mode)
  (form-update-visibility "group_filters" mode)
  (form-update-visibility "actions" mode)
;  (form-update-visibility "move_rule" mode)

  (path_dir_file visibility (not mode))
  (form-update-visibility "label_path" (not mode))
  (form-update-visibility "group_perm" (not mode))
  (form-update-visibility "check_file" (not mode))
)

(define fileselect (make-widget 'fileselect))
(define url (make-attribute 'url))
(define value (make-attribute 'value))
(define filter (make-attribute 'filter))
(define hints (make-attribute 'hints))


(define (ui-exit)
  (document:end))

(define (dir_or_file)
  (if (form-value "check_file")
    (path_dir_file hints "existing_file"
		   title (_"Select file")
		   filter "*")
    (path_dir_file hints "existing_file;directory;show_dirs_only"
		   title (_"Select directory")
		   filter "*.directory"))
)


;;; UI

width 800
height 600


(gridbox columns "10;100;10" align "top"
  (spacer)
  (gridbox columns "70;5;100" align "top"

	 (vbox
	   (label)
	   (checkbox (_ "Expert mode") name "expert_mode" align "left"))
	 (spacer)
	 (spacer)

	 (groupbox (_ "Rules")
		   (document:id rule_all (listbox name "all_rules"
						  columns 2
						  header (vector (_ "Rule") (_ "Status"))
						  row '#((rule . "") (check . "") )
						  ))
		   (hbox align "left"
			   (button (_ "Templates") (when clicked (template))))
	  )
	 (spacer)
	 (vbox
	     (gridbox columns "10;100"

		  (label (_ "Name"))
		  (document:id rule_name (edit name "name"))
;;///basic
		  (checkbox (_ "(File) Path") name "check_file")
		  (document:id path_dir_file (fileselect url "/"))
;;///
		  (label (_ "Rule") name "label_rule")
		  (combobox name "rule_list")
		  (groupbox (_ "Action") 
			    orientation "horizontal" 
			    colspan 2
			    name "actions"
			    (radio text "always" name "rule_action" value "always")
			    (radio text "newer" name "rule_action" value "newer"))
		  (label (_ "Syscall") name "label_syscall")
		  (edit name "rule_syscall"))
;;///basic
	     (groupbox (_ "Permissive")
		       name "group_perm"
		       colspan 2
		       (hbox
			 (checkbox (_ "read") name "perm_r")
			 (checkbox (_ "execute") name "perm_x"))
		       (hbox
			 (checkbox (_ "write") name "perm_w")
			 (checkbox (_ "change attribute") name "perm_a")))
;;///

	     (groupbox (_ "Filters") name "group_filters"
		       (document:id fil_list
				    (listbox columns 3
					     name "filter_list"
					     header (vector (_ "filter") (_ "check") (_ "value"))
					     row '#((label . "") (check . "") (summary . "")) ) )
		       (hbox
			 (document:id fil (combobox name "filters" align "left"))
			 (document:id fil_check (combobox name "condition" align "left"))
			 (document:id fil_val (edit name "filter_val"
						    (when return-pressed (change_filter)))) )

		       (gridbox columns "10;3;10"
				(hbox align "left"
				  (button (_ "Add filter") name "add_filter")
				  (button (_ "Change") name "change_filter" visibility #f))
				(spacer)
				(hbox align "right"
				  (button (_ "Clean list") name "clean_filter_list" align "left" visibility #f
					  (when clicked (fil_list rows-clear)))
				  (button (_ "Delete") name "delete_filter" visibility #f
					(when clicked (fil_list row-remove (fil_list current)))))) )
	     (spacer)
	     (checkbox (_ "Activate rule") name "active")
	     (spacer)
	     (hbox 
;	       	(hbox align "left" name "move_rule" visibility #f
;		  (label (_ "move rule in the list:"))
;		  (button (_ "Up") (when clicked (move_rule "up")))
;		  (button (_ "Down") (when clicked (move_rule "down"))))
	     (spacer)
		(hbox align "right;bottom"
		      (button (_ "Add Rule") name "add_rule"
			      (when clicked (new_rule "add")))
		      (button (_ "Change Rule") name "save_rule"
			      (when clicked (new_rule "change") (form-update-visibility "reload" #t) ))
		      (button (_ "Delete") name "del_rule"
			      (when clicked (delete_rule)) )) ))
	 (vbox
	   (spacer)
	   (hbox
	     (button (_ "Back") align "left;bottom" name "cancel"
		 (when clicked (frame:replace "/audit")))
	     (hbox align "center" name "reload" visibility #f
		 (label (_ "Please reload configuration of rules "))
		 (button (_ "Reload") (when clicked (woo-call "/audit/reload_rules") (form-update-visibility "reload" #f))))))
  )
)

;;
(document:root
  (when loaded
    (ui-init)
    (expert)
    (form-bind "all_rules" "change" config_rule)
    (form-bind "expert_mode" "change" config_rule)
    (form-bind "filter_list" "change" config_filter)
    (form-bind "change_filter" "click" change_filter)
    (form-bind "add_filter" "click" add_filter)
    (form-bind "expert_mode" "change" expert)
    (form-bind "active" "change" activate)

    (form-bind "check_file" "change" dir_or_file)
))
